/*
 *  Copyright (c) 2005 Sun Microsystems, Inc.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  3. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Sun Microsystems, Inc. for Project JXTA."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact Project JXTA at http://www.jxta.org.
 *
 *  5. Products derived from this software may not be called "JXTA",
 *  nor may "JXTA" appear in their name, without prior written
 *  permission of Sun.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *  ====================================================================
 *
 *  This software consists of voluntary contributions made by many
 *  individuals on behalf of Project JXTA.  For more
 *  information on Project JXTA, please see
 *  <http://www.jxta.org/>.
 *
 *  This license is based on the BSD license adopted by the Apache Foundation.
 *
 *  $Id: NetworkManager.java,v 1.5 2006/03/16 16:48:40 hamada Exp $
 */

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;

import contrib.rendezvous.ConfigurationFactory;
import net.jxta.document.Advertisement;
import net.jxta.exception.PeerGroupException;
import net.jxta.id.IDFactory;
import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupFactory;
import net.jxta.peergroup.PeerGroupID;
import net.jxta.rendezvous.RendezVousService;
import net.jxta.rendezvous.RendezvousEvent;
import net.jxta.rendezvous.RendezvousListener;

/**
 *  A simple and re-usable exmaple of starting and stopping a JXTA platform
 *
 *@author     Mohamed Abdelaziz (hamada)
 *@created    December 17, 2005
 */

public class NetworkManager implements RendezvousListener {

    private PeerGroup netPeerGroup = null;
    private boolean started = false;
    private boolean stopped = false;
    private RendezVousService rendezvous;
    private final static String connectLock = new String("connectLock");


    /**
     *  A simple and re-usable exmaple of starting and stopping a JXTA platform
     */
    public NetworkManager() { }


    /**
     *  Main method
     *
     *@param  args  Description of the Parameter
     */
    public static void main(String args[]) {
        NetworkManager manager = new NetworkManager();
        System.out.println("Starting NetworkManager ....");
        manager.start();
        PeerGroup netPG = manager.getNetPeerGroup();
        manager.waitForRendezvousConncection(10000);
        System.out.println("Good Bye ....");
        manager.stop();
    }


    /**
     *  returns a resource InputStream
     *
     *@param  resource         resource name
     *@return                  returns a resource InputStream
     *@exception  IOException  if an I/O error occurs
     */
    private InputStream getResourceInputStream(String resource) throws IOException {
        ClassLoader classLoader = NetworkManager.class.getClassLoader();
        return classLoader.getResourceAsStream(resource);
    }


    /**
     *  Creates and starts the JXTA NetPeerGroup using a platform configuration
     *  template. This class also registers a listener for rendezvous events
     */
    public synchronized void start() {
        if (started) {
            return;
        }
        try {
            File home = new File(System.getProperty("JXTA_HOME", ".jxta"));
            ConfigurationFactory.setHome(home);
            ConfigurationFactory.setPeerID(IDFactory.newPeerID(PeerGroupID.defaultNetPeerGroupID));
            ConfigurationFactory.setName("Network Manager");
            ConfigurationFactory.setDescription("Created by ConfigurationFactory");
            ConfigurationFactory.setMode(ConfigurationFactory.EDGE);
            ConfigurationFactory.setPrincipal("principal");
            ConfigurationFactory.setPassword("password");
            ConfigurationFactory.setTCPPortRange(9701, 9799);
            try {
                ConfigurationFactory.setRdvSeedingURI(new URI("http://rdv.jxtahosts.net/cgi-bin/rendezvous.cgi?2"));
                ConfigurationFactory.setRelaySeedingURI(new URI("http://rdv.jxtahosts.net/cgi-bin/relays.cgi?2"));
            } catch (java.net.URISyntaxException use) {
                use.printStackTrace();
            }
            Advertisement config = ConfigurationFactory.newPlatformConfig();
            try {
                // this always creates a new PlatformConfig, a false value does not
                // overwrite what exsits
                ConfigurationFactory.save(config, false);
            } catch (IOException io) {
                io.printStackTrace();
            }
            // create, and Start the default jxta NetPeerGroup
            netPeerGroup = PeerGroupFactory.newNetPeerGroup();
            System.out.println(netPeerGroup.getPeerID().getUniqueValue().toString());
            rendezvous = netPeerGroup.getRendezVousService();
            rendezvous.addListener(this);
            started = true;
        } catch (PeerGroupException e) {
            // could not instantiate the group, print the stack and exit
            System.out.println("fatal error : group creation failure");
            e.printStackTrace();
            System.exit(1);
        }
    }


    /**
     *  Stops and unrefrences the NetPeerGroup
     */
    public synchronized void stop() {
        if (stopped && !started) {
            return;
        }
        rendezvous.removeListener(this);
        netPeerGroup.stopApp();
        netPeerGroup.unref();
        netPeerGroup = null;
        stopped = true;
    }


    /**
     *  Gets the netPeerGroup object
     *
     *@return    The netPeerGroup value
     */
    public PeerGroup getNetPeerGroup() {
        return netPeerGroup;
    }


    /**
     *  Blocks until a connection to rendezvous node occurs
     *
     *@param  timeout  timeout in milliseconds
     */
    public void waitForRendezvousConncection(long timeout) {
        if (!rendezvous.isConnectedToRendezVous() || !rendezvous.isRendezVous()) {
            System.out.println("Waiting for Rendezvous Connection");
            try {
                if (!rendezvous.isConnectedToRendezVous()) {
                    synchronized(connectLock) {
                        connectLock.wait(timeout);
                    }
                }
                System.out.println("Connected to Rendezvous");
            } catch (InterruptedException e) {}
        }
    }


    /**
     *  rendezvousEvent the rendezvous event
     *
     *@param  event  rendezvousEvent
     */
    public void rendezvousEvent(RendezvousEvent event) {
        System.out.println(event.getType());
        if (event.getType() == event.RDVCONNECT ||
            event.getType() == event.RDVRECONNECT ||
            event.getType() == event.BECAMERDV) {
            synchronized (connectLock) {
                connectLock.notify();
            }
        }
    }
}

